@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with the GNU Shepherd.  If not, see <http://www.gnu.org/licenses/>.
+# along with the GNU Shepherd.  If not, see <https://www.gnu.org/licenses/>.
 
 shepherd --version
 herd --version
@@ -24,18 +24,20 @@ conf="t-conf-$$"
 log="t-log-$$"
 pid="t-pid-$$"
 service_pid="t-service-pid-$$"
+service_child_pid="t-service-child-pid-$$"
 service_log="t-service-log-$$"
 
 herd="herd -s $socket"
 
-trap "cat $log || true; rm -f $socket $conf $log $service_pid $service_log;
+trap "cat $log || true; rm -f $socket $conf $log $service_pid $service_child_pid $service_log;
       test -f $pid && kill \`cat $pid\` || true; rm -f $pid" EXIT
 
 cat > "$conf" <<EOF
 (use-modules (shepherd service timer) (srfi srfi-19))
 
 (define endless
-  "echo Started endless timer.; echo \$\$ > $PWD/$service_pid; sleep 500")
+  "echo Started endless timer.; echo \$\$ > $PWD/$service_pid; sleep 500 & \
+echo \$! > $PWD/$service_child_pid; wait")
 
 (define timers
   (list (service '(timer-with-command)
@@ -142,10 +144,12 @@ sleep 2
 grep "Started endless timer" "$log"
 $herd status endless-timer | grep "Started endless timer" # recent messages
 kill -0 $(cat "$service_pid")
+until kill -0 $(cat "$service_child_pid"); do sleep 0.2; done
 $herd status endless-timer | grep "Executing timer action"
 $herd status endless-timer | grep "Child process: $(cat "$service_pid")"
 $herd stop endless-timer
-kill -0 $(cat "$service_pid") && false
+kill -0 $(cat "$service_pid") && false	     # Main process and its entire
+kill -0 $(cat "$service_child_pid") && false # process group must be stopped.
 grep "Process $(cat "$service_pid") of timer 'endless-timer' terminated" "$log"
 
 $herd start timer-that-takes-too-long
